@@ -29,9 +29,9 @@ max.exon.ratio) {
     seg.id, min.variants.segment) {
     prior.M <- list(0.2,0.15,c(0.1,0.25),c(0.1,0.3),c(0.1,0.2,0.55))
     prior.M <- c(list(1), lapply(prior.M, function(x) c(x, 1-sum(x))))
-    max.M <- floor(Ci/2)
-    idx.germline <- test.num.copy+length(test.num.copy)+1
-    idx.somatic <- test.num.copy+1
+    max.M <- floor(Ci / 2)
+    idx.germline <- test.num.copy + length(test.num.copy) + 1
+    idx.somatic <- test.num.copy + 1
     variant.ok <- mapping.bias.ok & is.finite(apply(yy, 1, max))
     yys <- lapply(0:max.M, function(Mi) {
         for (i in test.num.copy) {
@@ -44,19 +44,19 @@ max.exon.ratio) {
                 yy[,idx.germline[i+1]] <- yy[,idx.germline[i+1]] + log(1-prior.K) - log(Cx + 1 - n.cases.germ)
 
                 # allow somatic mutations always have M=1
-                if (i<=1) {
+                if (i <= 1) {
                     yy[,idx.somatic[i+1]] <- yy[,idx.somatic[i+1]] + log(prior.K) - log(n.cases.somatic)
                 } else {
                     yy[,idx.somatic[i+1]] <- yy[,idx.somatic[i+1]] +
                         log(1-prior.K) - log(Cx + 1 - n.cases.somatic)
-                }    
+                } 
             } else {
-                yy[,idx.germline[i+1]] <- yy[,idx.germline[i+1]] + log(prior.K) -log(n.cases.germ)
-                yy[,idx.somatic[i+1]] <- yy[,idx.somatic[i+1]] + log(prior.K) - log(n.cases.somatic)
-            }    
+                yy[, idx.germline[i + 1]] <- yy[, idx.germline[i + 1]] + log(prior.K) - log(n.cases.germ)
+                yy[, idx.somatic[i + 1]] <- yy[, idx.somatic[i + 1]] + log(prior.K) - log(n.cases.somatic)
+            }
         }
         yy
-    })    
+    })
     # if not enough variants in segment, flag
     flag <- sum(variant.ok) < min.variants.segment
     if (!sum(variant.ok)) {
@@ -65,17 +65,22 @@ max.exon.ratio) {
         variant.ok <- rep(TRUE, length(variant.ok))
     }
 
-    likelihoodScores <- vapply(yys, function(x) { 
+    likelihoodScores <- vapply(yys, function(x) {
         sum(apply(x[variant.ok, , drop = FALSE], 1, max))
     }, double(1))
 
     best <- which.max(likelihoodScores)
-    # transform and scale 
+    # this should not happen...
+    if (!length(best)) {
+        flag <- TRUE
+        best <- 1
+    }
+    # transform and scale
     likelihoodScores <- exp(likelihoodScores - likelihoodScores[best])
-    posterior <- likelihoodScores[best]/sum(likelihoodScores, na.rm=TRUE)
-    if (is.na(posterior) || posterior < 0.5) flag <- TRUE 
+    posterior <- likelihoodScores[best] / sum(likelihoodScores, na.rm = TRUE)
+    if (is.na(posterior) || posterior < 0.5) flag <- TRUE
 
-    list(best=yys[[best]], M=test.num.copy[best], flag=flag, posterior=posterior)
+    list(best = yys[[best]], M = test.num.copy[best], flag = flag, posterior = posterior)
 }
 
 # calculate likelihood scores for all possible minor copy numbers
@@ -88,7 +93,7 @@ c(test.num.copy, round(opt.C))[i], prior.K, mapping.bias.ok, seg.id, min.variant
 .calcSNVLLik <- function(vcf, tumor.id.in.vcf, ov, p, test.num.copy, 
     C.likelihood, C, opt.C, median.C, snv.model, snv.lr, 
     sampleid = NULL, cont.rate = 0.01, prior.K, max.coverage.vcf, non.clonal.M,
-    model.homozygous=FALSE, error=0.001, max.mapping.bias=0.8, max.pon,
+    model.homozygous = FALSE, error = 0.001, max.mapping.bias = 0.8, max.pon,
     min.variants.segment) {
     
     .dbeta <- function(x, shape1, shape2, log, size, rho) dbeta(x=x, 
@@ -177,18 +182,18 @@ c(test.num.copy, round(opt.C))[i], prior.K, mapping.bias.ok, seg.id, min.variant
             p.ar[[4]] <- p.ar.cont.2 + log(prior.cont[idx])
 
             # homozygous state
-            p.ar[[5]] <- dbinom(round((1-ar_all[idx])*dp_all[idx]),size=round(dp_all[idx]), 
-                prob=error/3, log=TRUE) + priorHom + log(1-prior.somatic[idx])
-             
+            p.ar[[5]] <- dbinom(round((1 - ar_all[idx]) * dp_all[idx]),
+                size = round(dp_all[idx]), prob = error / 3, log = TRUE) +
+                priorHom + log(1 - prior.somatic[idx])
             do.call(cbind, p.ar)
         }))
-        
     })
     
     tmp <- lapply(seq_along(xx),function(i) .calcMsSegment(xx[[i]]$likelihoods, 
                test.num.copy, opt.C[seg.idx[i]], prior.K, 
-               mapping.bias.ok=info(vcf[xx[[i]]$vcf.ids])[[paste0(prefix, "MBB")]]>=max.mapping.bias,
-               seg.id=seg.idx[i], min.variants.segment))
+               mapping.bias.ok = info(vcf[xx[[i]]$vcf.ids])[[paste0(prefix, "MBB")]] >= max.mapping.bias &
+                                 info(vcf[xx[[i]]$vcf.ids])[[paste0(prefix, "MBB")]] <= (2 - max.mapping.bias),
+               seg.id = seg.idx[i], min.variants.segment))
 
     xx <- lapply(tmp, lapply, function(x) x$best)
     
@@ -196,7 +201,7 @@ c(test.num.copy, round(opt.C))[i], prior.K, mapping.bias.ok, seg.id, min.variant
         segmentValue <- sapply(seq_along(tmp), function(i) 
             tmp[[i]][[min(C[seg.idx[i]], max(test.num.copy))+1]][[field]])
         segmentValue <- unlist(sapply(seq_along(seg.idx), function(i) 
-            rep(segmentValue[i], sum(seg.idx[i]==queryHits(ov)))))
+            rep(segmentValue[i], sum(seg.idx[i] == queryHits(ov)))))
     }
     # Get segment M's for each SNV
     segment.M <- .extractValues(tmp, "M")
@@ -245,8 +250,7 @@ c(test.num.copy, round(opt.C))[i], prior.K, mapping.bias.ok, seg.id, min.variant
     posteriors$AR <- unlist(geno(vcf[vcf.ids])$FA[, tumor.id.in.vcf])
     posteriors$AR.ADJUSTED <- NA
     posteriors$MAPPING.BIAS <- info(vcf[vcf.ids])[[paste0(prefix, "MBB")]]
-    posteriors$AR.ADJUSTED <- posteriors$AR / posteriors$MAPPING.BIAS
-    posteriors$AR.ADJUSTED[posteriors$AR.ADJUSTED > 1] <- 1
+    posteriors$AR.ADJUSTED <- pmin(1, posteriors$AR / posteriors$MAPPING.BIAS)
     # Extract LOH
     posteriors$ML.LOH <- (posteriors$ML.M == posteriors$ML.C | 
         posteriors$ML.M == 0 | posteriors$ML.C == 1)
@@ -274,6 +278,7 @@ c(test.num.copy, round(opt.C))[i], prior.K, mapping.bias.ok, seg.id, min.variant
     rm.snv.posteriors <- apply(likelihoods, 1, max)
     idx.ignore <- rm.snv.posteriors == 0 | is.na(rm.snv.posteriors) |
         posteriors$MAPPING.BIAS < max.mapping.bias |
+        posteriors$MAPPING.BIAS > (2 - max.mapping.bias) |
         posteriors$start != posteriors$end
 
     posteriors$FLAGGED <- idx.ignore
@@ -332,7 +337,7 @@ c(test.num.copy, round(opt.C))[i], prior.K, mapping.bias.ok, seg.id, min.variant
 .checkParameters <- function(test.purity, min.ploidy, max.ploidy, 
     max.non.clonal, max.homozygous.loss, sampleid, prior.K, 
     prior.contamination, prior.purity, iterations, min.gof, model.homozygous, 
-    interval.file, log.ratio.calibration, test.num.copy) {
+    interval.file, log.ratio.calibration, test.num.copy, max.mapping.bias) {
     if (min(test.purity) <= 0 || max(test.purity) > 0.99) 
         .stopUserError("test.purity not within expected range.")
     if (min.ploidy <= 0 || max.ploidy <= 2) 
@@ -349,6 +354,7 @@ c(test.num.copy, round(opt.C))[i], prior.K, mapping.bias.ok, seg.id, min.variant
     .checkFraction(prior.K, "prior.K")
     .checkFraction(prior.contamination, "prior.contamination")
     .checkFraction(min.gof, "min.gof")
+    .checkFraction(max.mapping.bias, "max.mapping.bias")
 
     tmp <- sapply(prior.purity, .checkFraction, "prior.purity")
 
@@ -920,7 +926,7 @@ na.rm = TRUE)
         " invalid input data or parameters (PureCN ", 
         packageVersion("PureCN"), ").")
     flog.fatal(paste(strwrap(msg),"\n"))
-    stop(paste(strwrap(msg),"\n"), call.= FALSE)
+    stop(paste(strwrap(msg),collapse = "\n"), call.= FALSE)
 }
 .stopRuntimeError <- function(...) {
     msg <- paste(c(...), collapse="")
@@ -928,7 +934,7 @@ na.rm = TRUE)
         " invalid input data or parameters. Please report bug (PureCN ", 
         packageVersion("PureCN"), ").")
     flog.fatal(paste(strwrap(msg),"\n"))
-    stop(paste(strwrap(msg),"\n"), call.= FALSE)
+    stop(paste(strwrap(msg),collapse = "\n"), call.= FALSE)
 }
 .logHeader <- function(l) {
     flog.info(strrep("-", 60))
@@ -952,36 +958,50 @@ na.rm = TRUE)
     flog.info("Done.")
     flog.info(strrep("-", 60))
 }    
-.calcGCmetric <- function(gc_bias, coverage, on.target) {
+.calcGCmetric <- function(gc_bias, coverage, on.target, field = "average.coverage") {
     idx <- which(coverage$on.target==on.target)
     if (!length(idx)) return(NA)
-    gcbins <- split(coverage[idx]$average.coverage, 
-        gc_bias[idx] < 0.5)
+    gcbins <- split(mcols(coverage[idx])[,field], gc_bias[idx] < 0.5)
     mean(gcbins[[1]], na.rm=TRUE) / mean(gcbins[[2]], na.rm=TRUE) 
 }
-.checkGCBias <- function(normal, tumor, max.dropout, on.target=TRUE) {
+.checkGCBias <- function(normal, tumor, log.ratio, max.dropout, on.target = TRUE) {
+    # vector instead of tumor meta column provided
+   
+    if (is(log.ratio, "numeric") && length(log.ratio) == length(tumor)) {
+        tumor$ratio <- 2^(log.ratio)
+    } else {
+        .stopRuntimeError("tumor and log.ratio do not align in .checkGCBias")
+    }    
 
     gcMetricNormal <- .calcGCmetric(tumor$gc_bias, normal, on.target)
     gcMetricTumor <- .calcGCmetric(tumor$gc_bias, tumor, on.target)
+    gcMetricLogRatio <- .calcGCmetric(tumor$gc_bias, tumor, on.target, "ratio")
 
     if (is.na(gcMetricTumor)) return(FALSE)
 
-    flog.info("AT/GC dropout%s: %.2f (tumor), %.2f (normal). ", 
-        ifelse(on.target,""," (off-target regions)"), gcMetricTumor, gcMetricNormal)
-    if (gcMetricNormal < max.dropout[1] || 
+    flog.info("AT/GC dropout%s: %.2f (tumor), %.2f (normal), %.2f (coverage log-ratio). ", 
+        ifelse(on.target,""," (off-target regions)"), gcMetricTumor,
+        gcMetricNormal, gcMetricLogRatio)
+    # throw warning only when the gc bias extends to normalzed log-ratio
+    max.dropout.half <- sapply(max.dropout, function(x) x + (1 - x)/2)
+    if (gcMetricLogRatio < max.dropout.half[1] || 
+        gcMetricLogRatio > max.dropout.half[2]) {
+        flog.warn("High GC-bias in normalized tumor vs normal log2 ratio.")
+        return(TRUE)
+    } else if (gcMetricNormal < max.dropout[1] || 
         gcMetricNormal > max.dropout[2] ||
         gcMetricTumor  < max.dropout[1] ||
         gcMetricTumor  > max.dropout[2]) {
-        flog.warn("High GC-bias in normal or tumor. Is data GC-normalized?")
-        return(TRUE)
+        flog.info("High GC-bias in normal and/or tumor coverage.")
     }
     return(FALSE)
 }
 
-.gcGeneToCoverage <- function(interval.file, min.coverage) {
-    gc.data <- readCoverageFile(interval.file)
+.gcGeneToCoverage <- function(interval.file, min.coverage, min.total.counts) {
+    gc.data <- readIntervalFile(interval.file)
     gc.data$average.coverage <- min.coverage
     gc.data$coverage <- min.coverage * width(gc.data)
+    gc.data$counts <- Inf
     gc.data
 }
 
@@ -1020,11 +1040,12 @@ na.rm = TRUE)
     return(exon.lrs)
 }
 
-.estimateContamination <- function(pp,  max.mapping.bias=NULL, min.fraction.chromosomes=0.8) {
-    if (is.null(max.mapping.bias)) max.mapping.bias=0
+.estimateContamination <- function(pp,  max.mapping.bias = NULL, min.fraction.chromosomes = 0.8) {
+    if (is.null(max.mapping.bias)) max.mapping.bias <- 0
 
     idx <- pp$GERMLINE.CONTHIGH + pp$GERMLINE.CONTLOW > 0.5 & 
-        pp$MAPPING.BIAS >= max.mapping.bias
+        pp$MAPPING.BIAS >= max.mapping.bias &
+        pp$MAPPING.BIAS <= (2 - max.mapping.bias)
 
     if (!length(which(idx))) return(0)
     df <- data.frame(
@@ -1106,3 +1127,19 @@ na.rm = TRUE)
     }
     exon.lrs
 }
+
+# there are way too many bugs with this, so let's have one function we can more easily
+# debug
+.getSeqlevelsStyle <- function(x) {
+    if (is(x, "character")) {
+        sl <- x
+    } else {    
+        sl <- try(seqlevels(x))
+    }    
+    if (is(sl, "character")) {
+        style <- seqlevelsStyle(sl)
+        if ("Ensembl" %in% style) style <- "Ensembl"
+        return(style[1])    
+    }
+    seqlevelsStyle(x)
+}        
