@@ -421,6 +421,10 @@ runAbsoluteCN <- function(normal.coverage.file = NULL,
     if (!is.null(interval.file)) {
         tumor <- .addGCData(tumor, interval.file)
     }
+    if (is.null(centromeres) && !missing(genome)) {
+        centromeres <- .getCentromerePositions(centromeres, genome,
+            if (is.null(tumor)) NULL else .getSeqlevelsStyle(tumor))
+    }
 
     args.filterIntervals <- c(list(normal = normal, tumor = tumor,
         log.ratio = log.ratio, seg.file = seg.file,
@@ -437,6 +441,7 @@ runAbsoluteCN <- function(normal.coverage.file = NULL,
 
     # chr.hash is an internal data structure, so we need to do this separately.
     intervalsUsed <- .filterIntervalsChrHash(intervalsUsed, tumor, chr.hash)
+    intervalsUsed <- .filterIntervalsCentromeres(intervalsUsed, tumor, centromeres)
     intervalsUsed <- which(intervalsUsed)
     if (length(tumor) != length(normal) ||
         length(tumor) != length(log.ratio)) {
@@ -495,7 +500,8 @@ runAbsoluteCN <- function(normal.coverage.file = NULL,
         flog.info("Loading VCF...")
         vcf <- .readAndCheckVcf(vcf.file, genome = genome,
             DB.info.flag = DB.info.flag, POPAF.info.field = POPAF.info.field,
-            min.pop.af = min.pop.af, vcf.field.prefix = vcf.field.prefix)
+            min.pop.af = min.pop.af, error = error,
+            vcf.field.prefix = vcf.field.prefix)
 
         if (length(intersect(seqlevels(tumor), seqlevels(vcf))) < 1) {
             .stopUserError("Different chromosome names in coverage and VCF.")
@@ -572,10 +578,8 @@ runAbsoluteCN <- function(normal.coverage.file = NULL,
 
     flog.info("Sample sex: %s", sex)
     flog.info("Segmenting data...")
-    segProvided <- readSegmentationFile(seg.file, sampleid, model.homozygous = model.homozygous)
 
-    centromeres <- .getCentromerePositions(centromeres, genome,
-        if (is.null(vcf)) NULL else .getSeqlevelsStyle(vcf))
+    segProvided <- readSegmentationFile(seg.file, sampleid, model.homozygous = model.homozygous)
     
     args.segmentation <- c(list(normal = normal, tumor = tumor, log.ratio = log.ratio, 
         seg = segProvided, plot.cnv = plot.cnv,
