@@ -1,8 +1,8 @@
 context("poolCoverage")
 
-normal.coverage.file <- system.file("extdata", "example_normal.txt", 
+normal.coverage.file <- system.file("extdata", "example_normal.txt.gz", 
     package = "PureCN")
-normal2.coverage.file <- system.file("extdata", "example_normal2.txt", 
+normal2.coverage.file <- system.file("extdata", "example_normal2.txt.gz", 
     package = "PureCN")
 normal.coverage.files <- c(normal.coverage.file, normal2.coverage.file)
 
@@ -13,12 +13,12 @@ test_that("Example coverage is averaged", {
         pool$average.coverage)
     expect_equal(coverage[[1]]$coverage + coverage[[2]]$coverage, 
         pool$coverage)
-    pool2 <- poolCoverage(coverage, w=c(0.5, 0.5))
-    expect_equal((coverage[[1]]$coverage+coverage[[2]]$coverage)/2, 
+    pool2 <- poolCoverage(coverage, w = c(0.5, 0.5))
+    expect_equal((coverage[[1]]$coverage + coverage[[2]]$coverage) / 2,
         pool2$coverage)
 })
 
 test_that("Exceptions happend with wrong input", {
     coverage <- lapply(normal.coverage.files, readCoverageFile)
-    expect_error(poolCoverage(coverage, w=1:3), "different lengths")
+    expect_error(poolCoverage(coverage, w = seq(3)), "different lengths")
 })    
