@@ -1,11 +1,11 @@
 context("runAbsoluteCN")
 
-normal.coverage.file <- system.file("extdata", "example_normal.txt",
+normal.coverage.file <- system.file("extdata", "example_normal.txt.gz",
     package = "PureCN")
-normal2.coverage.file <- system.file("extdata", "example_normal2.txt",
+normal2.coverage.file <- system.file("extdata", "example_normal2.txt.gz",
     package = "PureCN")
 normal.coverage.files <- c(normal.coverage.file, normal2.coverage.file)
-tumor.coverage.file <- system.file("extdata", "example_tumor.txt",
+tumor.coverage.file <- system.file("extdata", "example_tumor.txt.gz",
     package = "PureCN")
 vcf.file <- system.file("extdata", "example.vcf.gz", package = "PureCN")
 interval.file <- system.file("extdata", "example_intervals.txt",
